'use client';
import clsx from 'clsx';
import { useRef, useState } from 'react';
import { mergeProps, useButton, useDisclosure, useFocusRing } from 'react-aria';
import { useDisclosureState } from 'react-stately';
import { Section, SectionBody, SectionHeader, SectionHeaderContent } from './StaticSection';
/**
 * To optimize rendering, most of the components are server-components,
 * and the interactiveness is mainly handled by a few key components like this one.
 */
export function InteractiveSection(props) {
    var _a, _b, _c;
    var id = props.id, className = props.className, _d = props.toggeable, toggeable = _d === void 0 ? false : _d, _e = props.defaultOpened, defaultOpened = _e === void 0 ? true : _e, _f = props.tabs, tabs = _f === void 0 ? [] : _f, _g = props.defaultTab, defaultTab = _g === void 0 ? (_a = tabs[0]) === null || _a === void 0 ? void 0 : _a.key : _g, header = props.header, overlay = props.overlay, _h = props.toggleIcon, toggleIcon = _h === void 0 ? '▶' : _h;
    var _j = useState(defaultTab), selectedTabKey = _j[0], setSelectedTab = _j[1];
    var selectedTab = (_b = tabs.find(function (tab) { return tab.key === selectedTabKey; })) !== null && _b !== void 0 ? _b : tabs[0];
    var state = useDisclosureState({
        defaultExpanded: defaultOpened,
    });
    var panelRef = useRef(null);
    var triggerRef = useRef(null);
    var _k = useDisclosure({}, state, panelRef), triggerProps = _k.buttonProps, panelProps = _k.panelProps;
    var buttonProps = useButton(triggerProps, triggerRef).buttonProps;
    var _l = useFocusRing(), isFocusVisible = _l.isFocusVisible, focusProps = _l.focusProps;
    return (<Section id={id} className={clsx('openapi-section', toggeable ? 'openapi-section-toggeable' : null, className, toggeable ? "".concat(className, "-").concat(state.isExpanded ? 'opened' : 'closed') : null)}>
            {header ? (<SectionHeader onClick={function () {
                if (toggeable) {
                    state.toggle();
                }
            }} className={className}>
                    <SectionHeaderContent className={className}>
                        {(selectedTab === null || selectedTab === void 0 ? void 0 : selectedTab.body) && toggeable ? (<button {...mergeProps(buttonProps, focusProps)} ref={triggerRef} className={clsx('openapi-section-toggle', "".concat(className, "-toggle"))} style={{
                    outline: isFocusVisible
                        ? '2px solid rgb(var(--primary-color-500) / 0.4)'
                        : 'none',
                }}>
                                {toggleIcon}
                            </button>) : null}
                        {header}
                    </SectionHeaderContent>
                    <div className={clsx('openapi-section-header-controls', "".concat(className, "-header-controls"))} onClick={function (event) {
                event.stopPropagation();
            }}>
                        {tabs.length > 1 ? (<select className={clsx('openapi-section-select', 'openapi-select', "".concat(className, "-tabs-select"))} value={(_c = selectedTab === null || selectedTab === void 0 ? void 0 : selectedTab.key) !== null && _c !== void 0 ? _c : ''} onChange={function (event) {
                    setSelectedTab(event.target.value);
                    state.expand();
                }}>
                                {tabs.map(function (tab) { return (<option key={tab.key} value={tab.key}>
                                        {tab.label}
                                    </option>); })}
                            </select>) : null}
                    </div>
                </SectionHeader>) : null}
            {(!toggeable || state.isExpanded) && (selectedTab === null || selectedTab === void 0 ? void 0 : selectedTab.body) ? (<SectionBody ref={panelRef} {...panelProps} className={className}>
                    {selectedTab === null || selectedTab === void 0 ? void 0 : selectedTab.body}
                </SectionBody>) : null}
            {overlay ? (<div className={clsx('openapi-section-overlay', "".concat(className, "-overlay"))}>
                    {overlay}
                </div>) : null}
        </Section>);
}
