'use client';
import clsx from 'clsx';
import { useCallback } from 'react';
import { useStore } from 'zustand';
import { getOrCreateTabStoreByKey } from './useSyncedTabsGlobalState';
function useMediaTypeState(data, defaultKey) {
    var method = data.method, path = data.path;
    var store = useStore(getOrCreateTabStoreByKey("media-type-".concat(method, "-").concat(path), defaultKey));
    if (typeof store.tabKey !== 'string') {
        throw new Error('Media type key is not a string');
    }
    return {
        mediaType: store.tabKey,
        setMediaType: useCallback(function (index) { return store.setTabKey(index); }, [store.setTabKey]),
    };
}
function useMediaTypeSampleIndexState(data, mediaType) {
    var method = data.method, path = data.path;
    var store = useStore(getOrCreateTabStoreByKey("media-type-sample-".concat(mediaType, "-").concat(method, "-").concat(path), 0));
    if (typeof store.tabKey !== 'number') {
        throw new Error('Example key is not a number');
    }
    return {
        index: store.tabKey,
        setIndex: useCallback(function (index) { return store.setTabKey(index); }, [store.setTabKey]),
    };
}
export function OpenAPIMediaTypeExamplesSelector(props) {
    var method = props.method, path = props.path, renderers = props.renderers;
    if (!renderers[0]) {
        throw new Error('No renderers provided');
    }
    var state = useMediaTypeState({ method: method, path: path }, renderers[0].mediaType);
    var selected = renderers.find(function (r) { return r.mediaType === state.mediaType; }) || renderers[0];
    return (<div className="openapi-codesample-selectors">
            <MediaTypeSelector state={state} renderers={renderers}/>
            <ExamplesSelector method={method} path={path} renderer={selected}/>
        </div>);
}
function MediaTypeSelector(props) {
    var renderers = props.renderers, state = props.state;
    if (renderers.length < 2) {
        return null;
    }
    return (<select className={clsx('openapi-select')} value={state.mediaType} onChange={function (e) { return state.setMediaType(e.target.value); }}>
            {renderers.map(function (renderer) { return (<option key={renderer.mediaType} value={renderer.mediaType}>
                    {renderer.mediaType}
                </option>); })}
        </select>);
}
function ExamplesSelector(props) {
    var method = props.method, path = props.path, renderer = props.renderer;
    var state = useMediaTypeSampleIndexState({ method: method, path: path }, renderer.mediaType);
    if (renderer.examples.length < 2) {
        return null;
    }
    return (<select className={clsx('openapi-select')} value={String(state.index)} onChange={function (e) { return state.setIndex(Number(e.target.value)); }}>
            {renderer.examples.map(function (example, index) { return (<option key={index} value={index}>
                    {example.example.summary || "Example ".concat(index + 1)}
                </option>); })}
        </select>);
}
export function OpenAPIMediaTypeExamplesBody(props) {
    var _a;
    var renderers = props.renderers, method = props.method, path = props.path;
    if (!renderers[0]) {
        throw new Error('No renderers provided');
    }
    var mediaTypeState = useMediaTypeState({ method: method, path: path }, renderers[0].mediaType);
    var selected = (_a = renderers.find(function (r) { return r.mediaType === mediaTypeState.mediaType; })) !== null && _a !== void 0 ? _a : renderers[0];
    if (selected.examples.length === 0) {
        return selected.element;
    }
    return <ExamplesBody method={method} path={path} renderer={selected}/>;
}
function ExamplesBody(props) {
    var _a;
    var method = props.method, path = props.path, renderer = props.renderer;
    var exampleState = useMediaTypeSampleIndexState({ method: method, path: path }, renderer.mediaType);
    var example = (_a = renderer.examples[exampleState.index]) !== null && _a !== void 0 ? _a : renderer.examples[0];
    if (!example) {
        throw new Error("No example found for index ".concat(exampleState.index));
    }
    return example.element;
}
