'use client';
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import { createContext, useContext, useRef, useState } from 'react';
import { mergeProps, useButton, useDisclosure, useFocusRing, useId } from 'react-aria';
import { useDisclosureGroupState, useDisclosureState, } from 'react-stately';
var DisclosureGroupStateContext = createContext(null);
/**
 * Display an interactive OpenAPI disclosure group.
 */
export function OpenAPIDisclosureGroup(props) {
    var icon = props.icon, groups = props.groups;
    var state = useDisclosureGroupState(props);
    return (<DisclosureGroupStateContext.Provider value={state}>
            {groups.map(function (group) { return (<DisclosureItem icon={icon} key={group.id} group={group}/>); })}
        </DisclosureGroupStateContext.Provider>);
}
function DisclosureItem(props) {
    var _a, _b, _c, _d, _e, _f;
    var icon = props.icon, group = props.group;
    var defaultId = useId();
    var id = group.id || defaultId;
    var groupState = useContext(DisclosureGroupStateContext);
    var isExpanded = (groupState === null || groupState === void 0 ? void 0 : groupState.expandedKeys.has(id)) || false;
    var state = useDisclosureState({
        isExpanded: isExpanded,
        onExpandedChange: function () {
            if (groupState) {
                groupState.toggleKey(id);
            }
        },
    });
    var panelRef = useRef(null);
    var triggerRef = useRef(null);
    var isDisabled = (groupState === null || groupState === void 0 ? void 0 : groupState.isDisabled) || !((_a = group.tabs) === null || _a === void 0 ? void 0 : _a.length) || false;
    var _g = useDisclosure(__assign(__assign({}, props), { isExpanded: isExpanded, isDisabled: isDisabled }), state, panelRef), triggerProps = _g.buttonProps, panelProps = _g.panelProps;
    var buttonProps = useButton(triggerProps, triggerRef).buttonProps;
    var _h = useFocusRing(), isFocusVisible = _h.isFocusVisible, focusProps = _h.focusProps;
    var defaultTab = ((_c = (_b = group.tabs) === null || _b === void 0 ? void 0 : _b[0]) === null || _c === void 0 ? void 0 : _c.id) || '';
    var _j = useState(defaultTab), selectedTabKey = _j[0], setSelectedTabKey = _j[1];
    var selectedTab = (_d = group.tabs) === null || _d === void 0 ? void 0 : _d.find(function (tab) { return tab.id === selectedTabKey; });
    return (<div className="openapi-disclosure-group" aria-expanded={state.isExpanded}>
            <div className="openapi-disclosure-group-header">
                <button slot="trigger" ref={triggerRef} {...mergeProps(buttonProps, focusProps)} disabled={isDisabled} style={{
            outline: isFocusVisible
                ? '2px solid rgb(var(--primary-color-500)/0.4)'
                : 'none',
        }} className="openapi-disclosure-group-trigger">
                    <div className="openapi-disclosure-group-icon">
                        {icon || (<svg viewBox="0 0 24 24" className="openapi-disclosure-group-icon">
                                <path d="m8.25 4.5 7.5 7.5-7.5 7.5"/>
                            </svg>)}
                    </div>

                    {group.label}
                </button>
                {group.tabs ? (<div className="openapi-disclosure-group-mediatype">
                        {((_e = group.tabs) === null || _e === void 0 ? void 0 : _e.length) > 1 ? (<select className="openapi-section-select openapi-select openapi-disclosure-group-tabs-select" onClick={function (event) { return event.stopPropagation(); }} value={selectedTab === null || selectedTab === void 0 ? void 0 : selectedTab.id} onChange={function (event) {
                    setSelectedTabKey(event.target.value);
                    state.expand();
                }}>
                                {group.tabs.map(function (tab) { return (<option key={tab.id} value={tab.id}>
                                        {tab.label}
                                    </option>); })}
                            </select>) : ((_f = group.tabs[0]) === null || _f === void 0 ? void 0 : _f.label) ? (<span>{group.tabs[0].label}</span>) : null}
                    </div>) : null}
            </div>

            {state.isExpanded && selectedTab && (<div className="openapi-disclosure-group-panel" ref={panelRef} {...panelProps}>
                    {selectedTab.body}
                </div>)}
        </div>);
}
