import clsx from 'clsx';
import { Markdown } from './Markdown';
import { OpenAPICodeSample } from './OpenAPICodeSample';
import { OpenAPIPath } from './OpenAPIPath';
import { OpenAPIResponseExample } from './OpenAPIResponseExample';
import { OpenAPISpec } from './OpenAPISpec';
import { resolveDescription } from './utils';
/**
 * Display an interactive OpenAPI operation.
 */
export function OpenAPIOperation(props) {
    var _a;
    var className = props.className, data = props.data, context = props.context;
    var operation = data.operation;
    var clientContext = {
        defaultInteractiveOpened: context.defaultInteractiveOpened,
        icons: context.icons,
        blockKey: context.blockKey,
    };
    return (<div className={clsx('openapi-operation', className)}>
            <div className="openapi-summary" id={operation.summary ? undefined : context.id}>
                {(operation.deprecated || operation['x-stability']) && (<div className="openapi-summary-tags">
                        {operation.deprecated && (<div className="openapi-deprecated">Deprecated</div>)}
                        {operation['x-stability'] && (<OpenAPIOperationStability stability={operation['x-stability']}/>)}
                    </div>)}
                {operation.summary
            ? context.renderHeading({
                deprecated: (_a = operation.deprecated) !== null && _a !== void 0 ? _a : false,
                stability: operation['x-stability'],
                title: operation.summary,
            })
            : null}
                <OpenAPIPath data={data} context={context}/>
            </div>
            <div className="openapi-columns">
                <div className="openapi-column-spec">
                    {operation['x-deprecated-sunset'] ? (<div className="openapi-deprecated-sunset openapi-description openapi-markdown">
                            This operation is deprecated and will be sunset on{' '}
                            <span className="openapi-deprecated-sunset-date">
                                {operation['x-deprecated-sunset']}
                            </span>
                            {'.'}
                        </div>) : null}
                    <OpenAPIOperationDescription operation={operation} context={context}/>
                    <OpenAPISpec data={data} context={clientContext}/>
                </div>
                <div className="openapi-column-preview">
                    <div className="openapi-column-preview-body">
                        <OpenAPICodeSample {...props}/>
                        <OpenAPIResponseExample {...props}/>
                    </div>
                </div>
            </div>
        </div>);
}
function OpenAPIOperationDescription(props) {
    var operation = props.operation;
    if (operation['x-gitbook-description-document']) {
        return (<div className="openapi-intro">
                {props.context.renderDocument({
                document: operation['x-gitbook-description-document'],
            })}
            </div>);
    }
    var description = resolveDescription(operation);
    if (!description) {
        return null;
    }
    return (<div className="openapi-intro">
            <Markdown className="openapi-description" source={description}/>
        </div>);
}
var stabilityEnum = {
    experimental: 'Experimental',
    alpha: 'Alpha',
    beta: 'Beta',
    stable: 'Stable',
};
function OpenAPIOperationStability(props) {
    var stability = props.stability;
    var foundStability = stabilityEnum[stability];
    if (!foundStability) {
        return null;
    }
    return (<div className={"openapi-stability openapi-stability-".concat(foundStability.toLowerCase())}>
            {foundStability}
        </div>);
}
