import { InteractiveSection } from './InteractiveSection';
import { Markdown } from './Markdown';
import { OpenAPISchemaName } from './OpenAPISchemaName';
import { resolveDescription } from './utils';
/**
 * Present securities authorization that can be used for this operation.
 */
export function OpenAPISecurities(props) {
    var securities = props.securities, context = props.context;
    if (securities.length === 0) {
        return null;
    }
    return (<InteractiveSection header="Authorizations" toggeable defaultOpened={false} toggleIcon={context.icons.chevronRight} className="openapi-securities" tabs={securities.map(function (_a) {
            var key = _a[0], security = _a[1];
            var description = resolveDescription(security);
            return {
                key: key,
                label: key,
                body: (<div className="openapi-schema">
                            <div className="openapi-schema-presentation">
                                {getLabelForType(security)}

                                {description ? (<Markdown source={description} className="openapi-securities-description"/>) : null}
                            </div>
                        </div>),
            };
        })}/>);
}
function getLabelForType(security) {
    var _a;
    switch (security.type) {
        case 'apiKey':
            return (<OpenAPISchemaName propertyName={(_a = security.name) !== null && _a !== void 0 ? _a : 'apiKey'} type="string" required/>);
        case 'http':
            if (security.scheme === 'basic') {
                return <OpenAPISchemaName propertyName="Authorization" type="string" required/>;
            }
            if (security.scheme === 'bearer') {
                var description = resolveDescription(security);
                return (<>
                        <OpenAPISchemaName propertyName="Authorization" type="string" required/>
                        {/** Show a default description if none is provided */}
                        {!description ? (<Markdown source={"Bearer authentication header of the form Bearer ".concat('&lt;token&gt;', ".")} className="openapi-securities-description"/>) : null}
                    </>);
            }
            return <OpenAPISchemaName propertyName="HTTP" required/>;
        case 'oauth2':
            return <OpenAPISchemaName propertyName="OAuth2" required/>;
        case 'openIdConnect':
            return <OpenAPISchemaName propertyName="OpenID Connect" required/>;
        default:
            // @ts-ignore
            return security.type;
    }
}
