'use client';
import { createContext, useContext, useEffect, useMemo, useRef, useState } from 'react';
import { Tab, TabList, TabPanel, Tabs } from 'react-aria-components';
import { useEventCallback } from 'usehooks-ts';
import { getOrCreateTabStoreByKey } from './useSyncedTabsGlobalState';
var OpenAPITabsContext = createContext(null);
function useOpenAPITabsContext() {
    var context = useContext(OpenAPITabsContext);
    if (!context) {
        throw new Error('OpenAPITabsContext is missing');
    }
    return context;
}
/**
 * The OpenAPI Tabs wrapper component.
 */
export function OpenAPITabs(props) {
    var _a, _b;
    var children = props.children, items = props.items, stateKey = props.stateKey;
    var _c = useState(function () {
        var _a, _b;
        if (stateKey && typeof window !== 'undefined') {
            var store = getOrCreateTabStoreByKey(stateKey);
            var tabKey_1 = store.getState().tabKey;
            if (tabKey_1) {
                return tabKey_1;
            }
        }
        return (_b = (_a = items[0]) === null || _a === void 0 ? void 0 : _a.key) !== null && _b !== void 0 ? _b : null;
    }), tabKey = _c[0], setTabKey = _c[1];
    var selectTab = useEventCallback(function (key) {
        if (!key || key === tabKey) {
            return;
        }
        var tab = items.find(function (item) { return item.key === key; });
        if (!tab) {
            return;
        }
        setTabKey(key);
    });
    var selectedTab = (_b = (_a = items.find(function (item) { return item.key === tabKey; })) !== null && _a !== void 0 ? _a : items[0]) !== null && _b !== void 0 ? _b : null;
    var cancelDeferRef = useRef(null);
    useEffect(function () {
        if (!stateKey) {
            return undefined;
        }
        var store = getOrCreateTabStoreByKey(stateKey);
        return store.subscribe(function (state) {
            var _a;
            (_a = cancelDeferRef.current) === null || _a === void 0 ? void 0 : _a.call(cancelDeferRef);
            cancelDeferRef.current = defer(function () { return selectTab(state.tabKey); });
        });
    }, [stateKey, selectTab]);
    useEffect(function () {
        return function () { var _a; return (_a = cancelDeferRef.current) === null || _a === void 0 ? void 0 : _a.call(cancelDeferRef); };
    }, []);
    var contextValue = useMemo(function () { return ({ items: items, selectedTab: selectedTab }); }, [items, selectedTab]);
    return (<OpenAPITabsContext.Provider value={contextValue}>
            <Tabs className="openapi-tabs" onSelectionChange={function (tabKey) {
            selectTab(tabKey);
            if (stateKey) {
                var store = getOrCreateTabStoreByKey(stateKey);
                store.setState({ tabKey: tabKey });
            }
        }} selectedKey={tabKey}>
                {children}
            </Tabs>
        </OpenAPITabsContext.Provider>);
}
var defer = function (fn) {
    var id = setTimeout(fn, 0);
    return function () { return clearTimeout(id); };
};
/**
 * The OpenAPI Tabs list component.
 * This component should be used as a child of the OpenAPITabs component.
 * It renders the list of tabs.
 */
export function OpenAPITabsList() {
    var items = useOpenAPITabsContext().items;
    return (<TabList className="openapi-tabs-list">
            {items.map(function (tab) { return (<Tab key={tab.key} id={tab.key} style={function (_a) {
                var isFocusVisible = _a.isFocusVisible;
                return ({
                    outline: isFocusVisible
                        ? '2px solid rgb(var(--primary-color-500)/0.4)'
                        : 'none',
                });
            }} className="openapi-tabs-tab">
                    {tab.label}
                </Tab>); })}
        </TabList>);
}
/**
 * The OpenAPI Tabs panels component.
 * This component should be used as a child of the OpenAPITabs component.
 * It renders the content of the selected tab.
 */
export function OpenAPITabsPanels() {
    var selectedTab = useOpenAPITabsContext().selectedTab;
    if (!selectedTab) {
        return null;
    }
    var key = selectedTab.key.toString();
    return (<TabPanel id={key} className="openapi-tabs-panel">
            <div className="openapi-tabs-body">{selectedTab.body}</div>
            {selectedTab.footer ? (<div className="openapi-tabs-footer">{selectedTab.footer}</div>) : null}
        </TabPanel>);
}
