import type { OpenAPIV3 } from '@gitbook/openapi-parser';
type JSONValue = string | number | boolean | null | JSONValue[] | {
    [key: string]: JSONValue;
};
type ScalarGetExampleFromSchemaOptions = NonNullable<Parameters<typeof getExampleFromSchema>[1]>;
type GenerateSchemaExampleOptions = Pick<ScalarGetExampleFromSchemaOptions, 'xml' | 'omitEmptyAndOptionalProperties' | 'mode'>;
/**
 * Generate a JSON example from a schema
 */
export declare function generateSchemaExample(schema: OpenAPIV3.SchemaObject, options?: GenerateSchemaExampleOptions): JSONValue | undefined;
/**
 * Generate an example for a media type.
 */
export declare function generateMediaTypeExamples(mediaType: OpenAPIV3.MediaTypeObject, options?: GenerateSchemaExampleOptions): OpenAPIV3.ExampleObject[];
/**
 * This function takes an OpenAPI schema and generates an example from it
 * Forked from : https://github.com/scalar/scalar/blob/main/packages/oas-utils/src/spec-getters/getExampleFromSchema.ts
 */
declare const getExampleFromSchema: (schema: Record<string, any>, options?: {
    /**
     * The fallback string for empty string values.
     * @default ''
     */
    emptyString?: string;
    /**
     * Whether to use the XML tag names as keys
     * @default false
     */
    xml?: boolean;
    /**
     * Whether to show read-only/write-only properties. Otherwise all properties are shown.
     * @default undefined
     */
    mode?: "read" | "write";
    /**
     * Dynamic values to add to the example.
     */
    variables?: Record<string, any>;
    /**
     * Whether to omit empty and optional properties.
     * @default false
     */
    omitEmptyAndOptionalProperties?: boolean;
}, level?: number, parentSchema?: Record<string, any>, name?: string) => any;
export {};
