import clsx from 'clsx';
import { OpenAPIDisclosureGroup } from '../OpenAPIDisclosureGroup';
import { OpenAPIRootSchema } from '../OpenAPISchemaServer';
import { Section, SectionBody } from '../StaticSection';
/**
 * Display OpenAPI Schemas.
 */
export function OpenAPISchemas(props) {
    var className = props.className, data = props.data, context = props.context, grouped = props.grouped;
    var schemas = data.schemas;
    var clientContext = {
        defaultInteractiveOpened: context.defaultInteractiveOpened,
        icons: context.icons,
        blockKey: context.blockKey,
    };
    if (!schemas.length) {
        return null;
    }
    return (<div className={clsx('openapi-schemas', className)}>
            <OpenAPIRootSchemasSchema grouped={grouped} schemas={schemas} context={clientContext}/>
        </div>);
}
/**
 * Root schema for OpenAPI schemas.
 * It displays a single model or a disclosure group for multiple schemas.
 */
function OpenAPIRootSchemasSchema(props) {
    var _a;
    var schemas = props.schemas, context = props.context, grouped = props.grouped;
    // If there is only one model and we are not grouping, we show it directly.
    if (schemas.length === 1 && !grouped) {
        var schema = (_a = schemas === null || schemas === void 0 ? void 0 : schemas[0]) === null || _a === void 0 ? void 0 : _a.schema;
        if (!schema) {
            return null;
        }
        return (<Section>
                <SectionBody>
                    <OpenAPIRootSchema schema={schema} context={context}/>
                </SectionBody>
            </Section>);
    }
    // If there are multiple schemas, we use a disclosure group to show them all.
    return (<OpenAPIDisclosureGroup allowsMultipleExpanded icon={context.icons.chevronRight} groups={schemas.map(function (_a) {
            var name = _a.name, schema = _a.schema;
            return ({
                id: name,
                label: (<div className="openapi-response-tab-content" key={"model-".concat(name)}>
                        <span className="openapi-response-statuscode">{name}</span>
                    </div>),
                tabs: [
                    {
                        id: 'model',
                        body: (<Section className="openapi-section-schemas">
                                <SectionBody>
                                    <OpenAPIRootSchema schema={schema} context={context}/>
                                </SectionBody>
                            </Section>),
                    },
                ],
            });
        })}/>);
}
